package net.jpos.tools.test;
import jpos.JposException;
import jpos.CashDrawer;
import jpos.CashDrawerConst;
import jpos.util.JposPropertiesConst;
import jpos.events.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.WindowEvent;

public class JposCashDrawerTest extends JFrame implements StatusUpdateListener {

    public static void main(String[] args) {
        try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (args.length < 2) {
            throw new RuntimeException("JposCashDrawerTest args Error!");
        }
//        JposCashDrawerTest frame = new JposCashDrawerTest("jpos.xml", "CashDrawer_USB");
        JposCashDrawerTest frame = new JposCashDrawerTest(args[0], args[1]);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = frame.getSize();
        frame.setResizable(false);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        frame.setVisible(true);
    }

    CashDrawer cd = new CashDrawer();

    JPanel contentPane;
    JPanel jPanel_reciept = new JPanel();
    TitledBorder titledBorder1;
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    GridBagLayout gridBagLayout2 = new GridBagLayout();
    JTextArea tipsArea = new JTextArea();
    private String logicName;

    /**
     * Constract "Frame"
     */
    public JposCashDrawerTest(String xmlFile, String logicName) {
        this.logicName = logicName;
        System.setProperty(JposPropertiesConst.JPOS_POPULATOR_FILE_PROP_NAME, xmlFile);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Form the component
     */
    private void jbInit() {
        contentPane = (JPanel) this.getContentPane();
        titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white, new Color(134, 134, 134)), "Receipt");
        contentPane.setLayout(gridBagLayout1);
        this.setSize(new Dimension(700, 280));
        this.setTitle("Hello JavaPOS");
        jPanel_reciept.setLayout(gridBagLayout2);
        jPanel_reciept.setBorder(titledBorder1);
        JButton openBtn = new JButton();
        openBtn.setText("Open Drawer");
        openBtn.addActionListener(e -> openClick());
        JButton statusBtn = new JButton();
        statusBtn.setText("Drawer Status");
        statusBtn.addActionListener(e->statusClick());
        JButton waitBtn = new JButton();
        waitBtn.setText("WaitForDrawerClose");
        waitBtn.addActionListener(e->waitClick());
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setBounds(10,10,10,10);
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        tipsArea.setLineWrap(true);
        scrollPane.setViewportView(tipsArea);
        contentPane.add(jPanel_reciept, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(15, 0, 0, 0), 20, 20));
        jPanel_reciept.add(openBtn, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 80, 0));
        jPanel_reciept.add(waitBtn, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 80, 0));
        jPanel_reciept.add(statusBtn, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 80, 0));
        contentPane.add(scrollPane, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(15, 0, 0, 0), 300, 193));
    }

    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            this.closing();
        } else if (e.getID() == WindowEvent.WINDOW_OPENED) {
            try {
                cd.addStatusUpdateListener(this);

//                ptr.open("JposPrinter_ethernet");
                cd.open(logicName);
                cd.claim(1);
                cd.setDeviceEnabled(true);
            } catch (JposException ex) {
                ex.printStackTrace();
                addMsg(ex.getMessage());
            }
        }
        // JavaPOS's code for Step1--END
    }

    //***********************Button*************************************************
    private void openClick() {
        try {
            cd.openDrawer();
        }catch (JposException e){
            e.printStackTrace();
        }
    }

    private void statusClick() {
        try {
            if (cd.getDrawerOpened())
            {
                addMsg("Drawer is OPENED");
            }else{
                addMsg("Drawer is CLOSED");
            }
        } catch (JposException e) {
            e.printStackTrace();
        }
    }

    private void waitClick() {
        try {
            cd.waitForDrawerClose(2000, 1000, 30000, 1000);
        } catch (JposException e) {
            e.printStackTrace();
        }
    }
    //***********************Method*************************************************

    /**
     * Outline     The code to finish a service.
     */
    void closing() {
        // JavaPOS's code for Step1
        try {
            //Cancel the device.
            cd.setDeviceEnabled(false);

            //Release the device exclusive control right.
            cd.release();

            //Finish using the device.
            cd.close();
        } catch (JposException ex) {
        }
        // JavaPOS's code for
        System.exit(0);
    }

    private void addMsg(String msg){
        tipsArea.insert(msg+"\n", 0);
    }

    @Override
    public void statusUpdateOccurred(StatusUpdateEvent statusUpdateEvent) {
        String msg;
        switch (statusUpdateEvent.getStatus()){
            case CashDrawerConst.CASH_SUE_DRAWERCLOSED:
                msg = "CASH_SUE_DRAWERCLOSED";
                break;
            case CashDrawerConst.CASH_SUE_DRAWEROPEN:
                msg = "CASH_SUE_DRAWEROPEN";
                break;
            default:
                msg = "Status:"+statusUpdateEvent.getStatus();
                break;
        }

        addMsg("StatusUpdateEvent->"+msg);
    }
}
